#ifndef _KGR_PATCH_BSC1184952_H
#define _KGR_PATCH_BSC1184952_H

#include <linux/types.h>
#include <linux/fs.h>

int kgr_patch_bsc1184952_init(void);
void kgr_patch_bsc1184952_cleanup(void);


struct fuse_io_priv;
struct fuse_attr;

int kgrp_fuse_dentry_revalidate(struct dentry *entry, unsigned int flags);
struct dentry *kgrp_fuse_lookup(struct inode *dir, struct dentry *entry,
				  unsigned int flags);
int kgrp_fuse_atomic_open(struct inode *dir, struct dentry *entry,
			    struct file *file, unsigned flags,
			    umode_t mode, int *opened);
int kgrp_fuse_mknod(struct inode *dir, struct dentry *entry, umode_t mode,
		      dev_t rdev);
int kgrp_fuse_mkdir(struct inode *dir, struct dentry *entry, umode_t mode);
int kgrp_fuse_symlink(struct inode *dir, struct dentry *entry,
			const char *link);
int kgrp_fuse_unlink(struct inode *dir, struct dentry *entry);
int kgrp_fuse_rmdir(struct inode *dir, struct dentry *entry);
int kgrp_fuse_rename2(struct inode *olddir, struct dentry *oldent,
			struct inode *newdir, struct dentry *newent,
			unsigned int flags);
int kgrp_fuse_link(struct dentry *entry, struct inode *newdir,
		     struct dentry *newent);
int kgrp_fuse_do_getattr(struct inode *inode, struct kstat *stat,
			   struct file *file);
int kgrp_fuse_permission(struct inode *inode, int mask);
int kgrp_fuse_readdir(struct file *file, struct dir_context *ctx);
const char *kgrp_fuse_follow_link(struct dentry *dentry, void **cookie);
int kgrp_fuse_do_setattr(struct dentry *dentry, struct iattr *attr,
		    struct file *file);
int kgrp_fuse_setattr(struct dentry *entry, struct iattr *attr);
int kgrp_fuse_getattr(struct vfsmount *mnt, struct dentry *entry,
			struct kstat *stat);
int kgrp_fuse_setxattr(struct dentry *entry, const char *name,
			 const void *value, size_t size, int flags);
ssize_t kgrp_fuse_getxattr(struct dentry *entry, const char *name,
			     void *value, size_t size);
ssize_t kgrp_fuse_listxattr(struct dentry *entry, char *list, size_t size);
int kgrp_fuse_open_common(struct inode *inode, struct file *file, bool isdir);
int kgrp_fuse_flush(struct file *file, fl_owner_t id);
int kgrp_fuse_fsync_common(struct file *file, loff_t start, loff_t end,
		      int datasync, int isdir);
int kgrp_fuse_readpage(struct file *file, struct page *page);
int kgrp_fuse_readpages(struct file *file, struct address_space *mapping,
			  struct list_head *pages, unsigned nr_pages);
ssize_t kgrp_fuse_perform_write(struct file *file,
				  struct address_space *mapping,
				  struct iov_iter *ii, loff_t pos);
ssize_t kgrp_fuse_file_write_iter(struct kiocb *iocb, struct iov_iter *from);
ssize_t kgrp___fuse_direct_read(struct fuse_io_priv *io,
				  struct iov_iter *iter,
				  loff_t *ppos);
ssize_t kgrp_fuse_direct_write_iter(struct kiocb *iocb, struct iov_iter *from);
int kgrp_fuse_writepages(struct address_space *mapping,
			   struct writeback_control *wbc);
long kgrp_fuse_ioctl_common(struct file *file, unsigned int cmd,
			unsigned long arg, unsigned int flags);
struct inode *kgrp_fuse_iget(struct super_block *sb, u64 nodeid,
			int generation, struct fuse_attr *attr,
			u64 attr_valid, u64 attr_version);

#define KGR_PATCH_BSC1184952_FUNCS					\
	KGR_PATCH_OBJ(fuse_dentry_revalidate,				\
		       kgrp_fuse_dentry_revalidate, "fuse"),		\
	KGR_PATCH_OBJ(fuse_lookup, kgrp_fuse_lookup, "fuse"),		\
	KGR_PATCH_OBJ(fuse_atomic_open, kgrp_fuse_atomic_open, "fuse"), \
	KGR_PATCH_OBJ(fuse_mknod, kgrp_fuse_mknod, "fuse"),		\
	KGR_PATCH_OBJ(fuse_mkdir, kgrp_fuse_mkdir, "fuse"),		\
	KGR_PATCH_OBJ(fuse_symlink, kgrp_fuse_symlink, "fuse"),	\
	KGR_PATCH_OBJ(fuse_unlink, kgrp_fuse_unlink, "fuse"),		\
	KGR_PATCH_OBJ(fuse_rmdir, kgrp_fuse_rmdir, "fuse"),		\
	KGR_PATCH_OBJ(fuse_rename2, kgrp_fuse_rename2, "fuse"),	\
	KGR_PATCH_OBJ(fuse_link, kgrp_fuse_link, "fuse"),		\
	KGR_PATCH_OBJ(fuse_do_getattr, kgrp_fuse_do_getattr, "fuse"),	\
	KGR_PATCH_OBJ(fuse_permission, kgrp_fuse_permission, "fuse"),	\
	KGR_PATCH_OBJ(fuse_readdir, kgrp_fuse_readdir, "fuse"),	\
	KGR_PATCH_OBJ(fuse_follow_link, kgrp_fuse_follow_link, "fuse"), \
	KGR_PATCH_OBJ(fuse_do_setattr, kgrp_fuse_do_setattr, "fuse"),	\
	KGR_PATCH_OBJ(fuse_setattr, kgrp_fuse_setattr, "fuse"),	\
	KGR_PATCH_OBJ(fuse_getattr, kgrp_fuse_getattr, "fuse"),	\
	KGR_PATCH_OBJ(fuse_setxattr, kgrp_fuse_setxattr, "fuse"),	\
	KGR_PATCH_OBJ(fuse_getxattr, kgrp_fuse_getxattr, "fuse"),	\
	KGR_PATCH_OBJ(fuse_listxattr, kgrp_fuse_listxattr, "fuse"),	\
	KGR_PATCH_OBJ(fuse_open_common, kgrp_fuse_open_common, "fuse"), \
	KGR_PATCH_OBJ(fuse_flush, kgrp_fuse_flush, "fuse"),		\
	KGR_PATCH_OBJ(fuse_fsync_common, kgrp_fuse_fsync_common,	\
		      "fuse"),						\
	KGR_PATCH_OBJ(fuse_readpage, kgrp_fuse_readpage, "fuse"),	\
	KGR_PATCH_OBJ(fuse_readpages, kgrp_fuse_readpages, "fuse"),	\
	KGR_PATCH_OBJ(fuse_perform_write, kgrp_fuse_perform_write,	\
		      "fuse"),						\
	KGR_PATCH_OBJ(fuse_file_write_iter, kgrp_fuse_file_write_iter,	\
		      "fuse"),						\
	KGR_PATCH_OBJ(__fuse_direct_read, kgrp___fuse_direct_read,	\
		      "fuse"),						\
	KGR_PATCH_OBJ(fuse_direct_write_iter,				\
		      kgrp_fuse_direct_write_iter, "fuse"),		\
	KGR_PATCH_OBJ(fuse_writepages, kgrp_fuse_writepages, "fuse"),	\
	KGR_PATCH_OBJ(fuse_ioctl_common, kgrp_fuse_ioctl_common,	\
		      "fuse"),						\
	KGR_PATCH_OBJ(fuse_iget, kgrp_fuse_iget, "fuse"),		\

#endif /* _KGR_PATCH_BSC1184952_H */
